@@ -1,6 +1,4 @@
 #!/usr/bin/python
-# -*- indent-tabs-mode: t -*-
-# vim: et:ts=4:sw=4:
 # This file is part of sp-endurance.
 #
 # Copyright (C) 2006-2009 by Nokia Corporation
@@ -228,11 +226,26 @@
 # - Process CPU Usage graph: show summary about the processes that we did not
 #   include in the graph.
 # 2009-06-01:
-# - System Load graph: fix division with zero with exactly identical data. This
-#   happened if user manually made another copy of one of the snapshot
+# - System Load graph: fix division with zero with exactly identical data.
+#   This happened if user manually made another copy of one of the snapshot
 #   directories.
+# 2009-10-15:
+# - Take last three xresource values, not ones from fixed offset.
+# 2009-10-26:
+# - Adapt to proc2csv providing whole command line
+# - Parse X client resource counts and show them in summary
+# - Parse ifconfig output and show network usage distribution between
+#   interfaces (network/radio usage has use-time implications)
+# - Generalize and move compressed file logging, opening and error
+#   handling to syslog_parse.py
+# 2009-10-29:
+# - Show program cmdline with acronym tag (mouse highlight)
+# - In memory graphs, handle process as same one regardless
+#   of program name/cmdline changes
+# - Get right fields from ifconfig and show network usage changes, not totals
+# 2009-11-02:
+# - Support Fremantle low memory limits scheme in addition to Diablo one
 # TODO:
-# - Have separate error_exit() function?
 # - Mark reboots more prominently also in report (<h1>):
 #   - dsme/stats/32wd_to -> HW watchdog reboot
 #   - dsme/stats/sw_rst -> SW watchdog reboots
@@ -272,19 +285,20 @@ EXAMPLES
         <TOOL_NAME> usecase/ usecase2/ > report.html
 """
 
-import sys, os, re, gzip
+import sys, os, re
 import syslog_parse as syslog
 
 # CSV field separator
 SEPARATOR = ','
 
-# these are HTML hex color values for different tables
+# these are HTML hex color values for different HTML tables
 class Colors:
     errors = "FDEEEE"
     disk = "EEFDFD"
     memory = "EEEEFD"
     threads = "CFEFEF"
-    xres = "FDEEFD"
+    xres_mem = "FDEEFD"
+    xres_count = "EEDDEE"
     fds = "FDFDEE"
     shm = "EEEEEE"
     kernel = "EFFDF0"
@@ -299,13 +313,14 @@ bar2colors = (bar1colors[0], "DE2821", "E0673E", "EAB040", "FBE84A")
 #      red, blue, light blue, magenta, light green
 bar3colors = (bar2colors[1], bar1colors[1], "4265FF", bar1colors[0], bar1colors[2])
 
+
 # --------------------- SMAPS data parsing --------------------------
 
 # address range, access rights, page offset, major:minor, inode, mmap()ed item
 smaps_mmap = re.compile("^[-0-9a-f]+ ([-rwxps]+) [0-9a-f]+ [:0-9a-f]+ \d+ *(|[^ ].*)$")
 
 # data from sp_smaps_snapshot
-def parse_smaps(filename):
+def parse_smaps(file):
     """
     Parse SMAPS and return (smaps, private_code):
       'smaps'        : Per PID dict with keys: private_dirty, swap, pss, rss
@@ -315,22 +330,7 @@ def parse_smaps(filename):
                        system.
     Everything is in kilobytes.
     """
-    if filename.endswith(".gz"):
-        # Unfortunately the python gzip module is slow. Using /bin/zcat and
-        # popen() gives 2-3x speed up.
-        if os.system("which zcat >/dev/null") == 0:
-            file = os.popen("zcat %s" % filename)
-        else:
-            file = gzip.open(filename, "r")
-    elif filename.endswith(".lzo"):
-        if os.system("which lzop >/dev/null") == 0:
-            file = os.popen("lzop -dc %s" % filename)
-        else:
-            parse_error(write, "ERROR: syslog file '%s' was compressed with lzop, but decompression program not available" % filename)
-            sys.exit(1)
-    else:
-        file = open(filename, "r")
-    private_code = code = idx = 0
+    private_code = code = pid = idx = 0
     smaps = {}
     while 1:
         try:
@@ -421,23 +421,35 @@ def get_filesystem_usage(file):
     return mounts
 
 
-def get_xclient_memory(file):
+def get_xres_usage(file):
     "reads X client resource usage, return command hash of total X mem usage"
-    clients = {}
+    xres_mem = {}
+    xres_count = {}
     while 1:
         line = file.readline().strip()
         if not line:
             break
-        mem,pid,name = line.split(',')[8:11]
-        if mem[-1] == 'B':
+        
+        # last three columns are the most interesting ones
+        mem,pid,name = line.split(',')[-3:]
+        if pid[-1] != 'B' and mem[-1] == 'B':
             mem = int(mem[:-1])
         else:
             sys.stderr.write("Error: X resource total memory value not followed by 'B':\n  %s\n" % line)
             sys.exit(1)
         # in KBs, check on clients taking > 1KB
         if mem >= 1024:
-            clients[name] = mem/1024
-    return clients
+            xres_mem[name] = mem/1024
+        
+        count = 0
+        # resource base, counts of resources, their memory usages, PID, name
+        cols = line.split(',')
+        for i in range(1, len(cols) - 3):
+            if cols[i][-1] != 'B':
+                count += int(cols[i])
+        xres_count[name] = count
+
+    return (xres_mem, xres_count)
 
 
 def get_process_info(file, headers):
@@ -509,20 +521,23 @@ def get_commands_and_fd_counts(file):
     returns pid hashes of command names and fd counts"""
     commands = {}
     fd_counts = {}
+    cmdlines = {}
     while 1:
         line = file.readline().strip()
         if not line:
             break
-        pid,fds,name = line.split(',')
-        commands[pid] = name
-        fd_counts[pid] = int(fds)
-    return (commands,fd_counts)
+        cols = line.split(',')
+        pid = cols[0]
+        fd_counts[pid] = int(cols[1])
+        # need to handle command lines with commas
+        cmdline = ",".join(cols[2:])
+        cmdlines[pid] = cmdline
+        commands[pid] = os.path.basename(cmdline.split(' ')[0])
+    return (commands,fd_counts,cmdlines)
 
 
-def parse_proc_stat(filename):
+def parse_proc_stat(file):
     "Parses relevant data from /proc/stat"
-    file = open(filename)
-    if not file: return None
     stat = {}
     # CPU: take everything except "steal" and "guest", which are some
     # virtualization related counters, obviously not useful in our case.
@@ -623,12 +638,9 @@ def skip_to_next_header(file):
             return line.strip()
 
 
-def parse_csv(filename):
+def parse_csv(file, filename):
     "Parses interesting information from the endurance measurement CSV file"
     data = {}
-    file = open(filename)
-    # filename without the extension
-    data['basedir'] = os.path.dirname(filename)
     
     # Check that file is generated with correct script so that
     # we can trust it's format and order of rows & fields:
@@ -664,14 +676,17 @@ def parse_csv(filename):
     # low memory limits
     skip_to(file, "lowmem_")
     mem = file.readline().split(',')
-    if len(mem) == 3:
+    if len(mem) in (3, 6):
         data['limitlow'] = int(mem[0])
         data['limithigh'] = int(mem[1])
         data['limitdeny'] = int(mem[2])
+        if len(mem) == 6:
+            data['limitlowpages'] = int(mem[3])
+            data['limithighpages'] = int(mem[4])
+            data['limitdenypages'] = int(mem[5])
     else:
         # not fatal as lowmem stuff is not in standard kernel
         sys.stderr.write("\nWarning: CSV file '%s' lowmem limits are missing!\n" % filename)
-        data['limitlow'] = data['limithigh'] = data['limitdeny'] = 0
 
     # get shared memory segment counts
     skip_to(file, "Shared memory segments")
@@ -684,7 +699,7 @@ def parse_csv(filename):
 
     # get the process FD usage
     skip_to(file, "PID,FD count,Command")
-    data['commands'], data['fdcounts'] = get_commands_and_fd_counts(file)
+    data['commands'], data['fdcounts'], data['cmdlines'] = get_commands_and_fd_counts(file)
     
     # get process statistics
     headers = skip_to(file, "Name,State,")
@@ -702,16 +717,42 @@ def parse_csv(filename):
         sys.exit(2)
 
     # get the X resource usage
-    data['xclients'] = get_xclient_memory(file)
+    data['xclient_mem'], data['xclient_count'] = get_xres_usage(file)
     
     # get the file system usage
     skip_to(file, "Filesystem")
     data['mounts'] = get_filesystem_usage(file)
     
-    # alles clar
-    file.close()
     return data
 
+# ------------------- ifconfig parsing -----------------------
+
+def parse_ifconfig(file):
+    """reads interface = [send,receive] information from ifocnfig output"""
+    #regex = re.compile("packets:([0-9]+) errors:([0-9]+) dropped:([0-9]+)")
+    regex = re.compile("RX bytes:([0-9]+) [^:]*TX bytes:([0-9]+)")
+    transfers = {}
+    interface = None
+    rbytes = tbytes = 0
+    while 1:
+        line = file.readline()
+        if not line:
+            break
+        if line[0] > ' ':
+            if interface and interface != "lo":
+                transfers[interface] = int(rbytes) + int(tbytes)
+            interface = line.split()[0]
+            continue
+        line = line.strip()
+        if line.startswith("RX bytes"):
+            match = regex.search(line)
+            if match:
+                rbytes,tbytes = match.groups()
+    if interface and interface != "lo":
+        transfers[interface] = int(rbytes) + int(tbytes)
+    return transfers
+
+
 # --------------------- HTML output ---------------------------
 
 def get_pids_from_procs(processes, commands):
@@ -891,7 +932,7 @@ def output_errors(idx, run1, run2):
 
 
 def output_data_links(run):
-    "output links to all collected data"
+    "output links to all collected uncompressed data"
     basedir = run['basedir']
     print "<h4>For more details on...</h4>"
     print "<ul>"
@@ -1003,7 +1044,7 @@ def output_run_diffs(idx1, idx2, data, do_summary):
             return
         # Scale the graphics to the largest CPU usage value.
         divisor = float(sum(diffs[0][1:3]))
-        output_memory_graph_table(\
+        output_graph_table(\
             ("Command[Pid]:", "<font color=%s>system</font> / <font color=%s>user</font>" % bar3colors[0:2], "CPU Usage:"),
             bar3colors[0:2],\
             [(x[0], (x[1]/divisor, x[2]/divisor),\
@@ -1073,9 +1114,13 @@ def output_run_diffs(idx1, idx2, data, do_summary):
         print "<p>No SMAPS data for process private memory usage available."
     
     # process X resource usage changes
-    diffs = get_usage_diffs(run1['xclients'], run2['xclients'])
-    output_diffs(diffs, "X resource usage", "X client", " kB",
-                    Colors.xres, do_summary)
+    diffs = get_usage_diffs(run1['xclient_mem'], run2['xclient_mem'])
+    output_diffs(diffs, "X resource memory usage", "X client", " kB",
+                 Colors.xres_mem, do_summary)
+    if do_summary:
+        diffs = get_usage_diffs(run1['xclient_count'], run2['xclient_count'])
+        output_diffs(diffs, "X resource count", "X client", "",
+                     Colors.xres_count, do_summary)
     
     # FD count changes
     diffs = get_pid_usage_diffs(run2['commands'], run2['processes'],
@@ -1182,7 +1227,7 @@ may appear under multiple error types.
 
 # ------------------- output memory graphs -------------------------
 
-def output_memory_graph_table(titles, colors, data):
+def output_graph_table(titles, colors, data):
     "outputs memory usage bars for given (name, (values), tex(t)) tupple array"
     width = 640 # total width of the graph bars
     print "<table><tr>"
@@ -1219,9 +1264,11 @@ def output_apps_memory_graphs(cases):
     smaps_available = 0
     rounds = 0
     data = {}
+    pidinfo = {}
     # get names and pids
     for testcase in cases:
         commands = testcase['commands']
+        cmdlines = testcase['cmdlines']
         processes = testcase['processes']
         for process in processes.values():
             pid = process['Pid']
@@ -1231,15 +1278,12 @@ def output_apps_memory_graphs(cases):
             if not pid_is_main_thread(pid, commands, processes):
                 continue
             name = commands[pid]
-            namepid = (name, pid)
-            if namepid not in data:
-                data[namepid] = {}
             try:
                 process['SMAPS_PRIVATE_DIRTY'] = testcase['smaps'][pid]['private_dirty']
                 smaps_available = 1
             except KeyError:
                 if 'smaps' in testcase:
-                    syslog.parse_error(sys.stdout.write, "WARNING: SMAPS data missing for %s[%s]" % namepid)
+                    syslog.parse_error(sys.stdout.write, "WARNING: SMAPS data missing for %s[%s]" % (name,pid))
             try: process['SMAPS_SWAP'] = testcase['smaps'][pid]['swap']
             except KeyError: pass
             try: process['SMAPS_PSS'] = testcase['smaps'][pid]['pss']
@@ -1248,7 +1292,11 @@ def output_apps_memory_graphs(cases):
             except KeyError: pass
             try: process['SMAPS_SIZE'] = testcase['smaps'][pid]['size']
             except KeyError: pass
-            data[namepid][rounds] = process
+            if pid not in data:
+                data[pid] = {}
+                pidinfo[pid] = {}
+            data[pid][rounds] = process
+            pidinfo[pid] = (name, cmdlines[pid])
         rounds += 1
 
     # get largest size for any of the namepids, get largest rss
@@ -1262,17 +1310,17 @@ def output_apps_memory_graphs(cases):
     #
     sizes = []
     largest_size = 0
-    for namepid in data:
+    for pid in data:
         changerounds = pidrounds = 0
         max_size = max_dirty = max_swap = 0
         min_size = min_dirty = min_swap = 512*1024
         for idx in range(rounds):
-            if idx in data[namepid]:
-                try:    dirty = data[namepid][idx]['SMAPS_PRIVATE_DIRTY']
-                except: dirty = data[namepid][idx]['VmRSS']
-                try:    size  = data[namepid][idx]['SMAPS_SIZE']
-                except: size  = data[namepid][idx]['VmSize']
-                try:    swap  = data[namepid][idx]['SMAPS_SWAP']
+            if idx in data[pid]:
+                try:    dirty = data[pid][idx]['SMAPS_PRIVATE_DIRTY']
+                except: dirty = data[pid][idx]['VmRSS']
+                try:    size  = data[pid][idx]['SMAPS_SIZE']
+                except: size  = data[pid][idx]['VmSize']
+                try:    swap  = data[pid][idx]['SMAPS_SWAP']
                 except: swap  = 0
                 min_dirty = min(dirty, min_dirty)
                 max_dirty = max(dirty, max_dirty)
@@ -1292,13 +1340,13 @@ def output_apps_memory_graphs(cases):
                 swap_and_dirty_change = (float)((max_dirty+min_swap) - (min_dirty+max_swap)) / (max_dirty+min_swap) / pidrounds
             else:
                 if smaps_available:
-                    syslog.parse_error(sys.stdout.write, "WARNING: no SMAPS dirty for %s[%s]. Disable swap and try again\n\t(SMAPS doesn't work properly with swap)" % namepid)
+                    syslog.parse_error(sys.stdout.write, "WARNING: no SMAPS dirty for %s[%s]. Disable swap and try again\n\t(SMAPS doesn't work properly with swap)" % (pid, pidinfo[pid][0]))
                 swap_and_dirty_change = 0
             size_change = (float)(max_size - min_size) / max_size / pidrounds
             # if >0.2% memory change per round in dirty or Size, or
             # size changes on more than half of the rounds, add to list
             if swap_and_dirty_change > 0.002 or size_change > 0.002 or 2*changerounds > pidrounds:
-                sizes.append((max_dirty,namepid))
+                sizes.append((max_dirty,pid))
         if max_size > largest_size:
             largest_size = max_size
     largest_size = float(largest_size)
@@ -1309,9 +1357,9 @@ def output_apps_memory_graphs(cases):
     # then sort according to names
     orders = []
     for size in sizes:
-        namepid = size[1]
-        # sorting order is: name, first round for pid, pid
-        orders.append((namepid[0], min(data[namepid].keys()), namepid[1]))
+        pid = size[1]
+        # data and sorting order is: name, first round for pid, pid
+        orders.append((pidinfo[pid][0], min(data[pid].keys()), pid))
     del(sizes)
     orders.sort()
     
@@ -1321,14 +1369,12 @@ def output_apps_memory_graphs(cases):
 changes during tests are listed.  If a process has same name and size
 as its parent, it's assumed to be a thread and ignored.
 
-<p>Note: Memory is not any more accounted as dirty if it's swapped out
-(even when it's paged back in), this seems like a kernel (2.6.21) bug.
-RSS can decrease if device is just running low on memory because
+<p>Note: RSS can decrease if device is just running low on memory because
 kernel can just discard unmodified/unused pages. Size tells amount of
 all virtual allocations and memory maps of a process, so it might not
 have any relation to real process memory usage. However, it can show
 leaks which cause process eventually to run out of (2GB) address space
-(e.g. if it's not collecting thread resources).
+(e.g. if it's not collecting joinable thread resources).
 """
 
     # LEGEND
@@ -1345,9 +1391,12 @@ leaks which cause process eventually to run out of (2GB) address space
 """ % (bar2colors[3], bar2colors[4])
 
     for order in orders:
-        namepid = (order[0],order[2])
-        process = data[namepid]
-        print "<h4><i>%s [%s]</i></h4>" % namepid
+        name = order[0]
+        pid = order[2]
+        cmdline = pidinfo[pid][1].replace('"', "&quot;")
+        print '<h4><i><acronym title="%s">%s</acronym> [%s]</i></h4>' % (cmdline, name, pid)
+        process = data[pid]
+        namepid = (name, pid)
         text = ''
         prev_idx = 0
         prev_text = ""
@@ -1389,18 +1438,14 @@ leaks which cause process eventually to run out of (2GB) address space
                 barwidth_size  = size/largest_size
                 #  ___________________________________________________
                 # |      |    ____________________     |              |
-                # |      |   |                    |    |              |
                 # |      |   |  _______________   |    |              |
                 # |      |   | |               |  |    |              |
-                # |      |   | | Private Dirty |  |    |              |
+                # | SWAP |   | | Private Dirty |  |    |              |
                 # |      |   | |_______________|  |    |              |
-                # | SWAP |   |                    |    |              |
                 # |      |   |        PSS         |    |              |
                 # |      |   |____________________|    |              |
-                # |      |                             |              |
                 # |      |            RSS              |              |
                 # |______|_____________________________|              |
-                # |                                                   |
                 # |                   Size                            |
                 # |___________________________________________________|
                 #
@@ -1434,7 +1479,7 @@ leaks which cause process eventually to run out of (2GB) address space
             titles[2] = "" #Swap
             titles[3] = "" #Dirty
             titles[4] = "" #PSS
-        output_memory_graph_table(titles, bar2colors, columndata)
+        output_graph_table(titles, bar2colors, columndata)
 
 
 def output_system_load_graphs(data):
@@ -1471,7 +1516,7 @@ def output_system_load_graphs(data):
         idx += 1
         prev = testcase
     titles = ("Test-case:", "system load:", "CPU usage-%:")
-    output_memory_graph_table(titles, bar3colors, entries)
+    output_graph_table(titles, bar3colors, entries)
     if reboots:
         text = '<p>Reboots occured during rounds:'
         for r in reboots:
@@ -1487,6 +1532,80 @@ def output_system_load_graphs(data):
     print '</table>'
 
 
+def output_network_use_graphs(data):
+    interfaces = {}
+    # collect interfaces
+    for testcase in data:
+        for face in testcase['transfers']:
+            if face not in interfaces and testcase['transfers'][face] > 0:
+                interfaces[face] = []
+    faces = interfaces.keys()
+    if not faces:
+        print "<p>Only local or no interfaces up when measurements were taken."
+        return
+
+    # collect test round data per interfaces
+    faces.sort()
+    for testcase in data:
+        for face in faces:
+            if face in testcase['transfers']:
+                interfaces[face].append(testcase['transfers'][face])
+            else:
+                interfaces[face].append(0)
+    
+    # arrange values shown as numbers and used for bar sizes into rounds
+    previous = []
+    for face in faces:
+        previous.append(interfaces[face][0])
+    prevrange = range(len(previous))
+    # ...with max bar size to use as scale
+    scale = 0
+    rounds = []
+    for r in range(1, len(data)):
+        valdiff = []
+        bardiff = []
+        current = []
+        for face in faces:
+            current.append(interfaces[face][r])
+        for i in prevrange:
+            if current[i]:
+                diff = current[i] - previous[i]
+                valdiff.append(diff)
+                if diff > 0:
+                    bardiff.append(diff)
+                else:
+                    # interface down and up, show as zero (bar size cannot be negative)
+                    bardiff.append(0)
+            else:
+                # interface down
+                valdiff.append(0)
+                bardiff.append(0)
+        previous = current
+        total = sum(bardiff)
+        if total > scale:
+            scale = total
+        rounds.append((bardiff,valdiff))
+    scale = float(scale)
+    
+    # create table
+    print '<p>Network interface usage distribution during the execution of test cases.'
+    print '<p>'
+    idx = 0
+    entries = []
+    for b,v in rounds:
+        idx += 1
+        bars = [x/scale for x in b]
+        vals = ["%dkB" % (x/1024) for x in v]
+        entries.append(('Test round %02d:' % idx, bars, vals))
+    titles = ["Test-case:", "network usage:"] + ["%s:" % x for x in faces]
+    output_graph_table(titles, bar1colors[:len(faces)], entries)
+    # Legend
+    print '<table><tr><th><th align="left">Legend:'
+    for i in range(len(faces)):
+        print '<tr><td bgcolor="%s" height=16 width=16><td>%s' % (bar1colors[i], faces[i])
+    print '</table>'
+
+
 def output_system_memory_graphs(data):
     "outputs memory graphs bars for the system"
     idx = 0
@@ -1507,21 +1626,28 @@ def output_system_memory_graphs(data):
 
         # amount of memory in the device (float for calculations)
         mem_total = float(testcase['ram_total'] + testcase['swap_total'])
-        # memory usage %-limit after which apps are bg-killed
-        mem_low = testcase['limitlow']
-        # memory usage %-limit after which apps refuse certain operations
-        mem_high = testcase['limithigh']
-        # memory usage %-limit after which kernel denies app allocs
-        mem_deny = testcase['limitdeny']
         
-        if mem_low + mem_high + mem_deny > 0:
+        # Fremantle low mem limits?
+        if 'limitlowpages' in testcase:
+            # limit is given as available free memory in pages
+            mem_low  = mem_total - 4*testcase['limitlowpages']
+            mem_high = mem_total - 4*testcase['limithighpages']
+            mem_deny = mem_total - 4*testcase['limitdenypages']
+        # Diablo low mem limits?
+        elif 'limitlow' in testcase:
             # convert percentages to real memory values
-            mem_low = mem_total * mem_low / 100
-            mem_high = mem_total * mem_high / 100
-            mem_deny = mem_total * mem_deny / 100
-        else:
+            percent = 0.01 * mem_total
+            # memory usage %-limit after which apps are bg-killed
+            mem_low  = testcase['limitlow'] * percent
+            # memory usage %-limit after which apps refuse certain operations
+            mem_high = testcase['limithigh'] * percent
+            # memory usage %-limit after which kernel denies app allocs
+            mem_deny = testcase['limitdeny'] * percent
+        # valid mem low limits?
+        if mem_low + mem_high + mem_deny <= 0:
             mem_low = mem_high = mem_deny = mem_total
             sys.stderr.write("Warning: low memory limits are zero -> disabling\n")
+        
         mem_used = testcase['ram_used'] + testcase['swap_used']
         mem_free = testcase['ram_free'] + testcase['swap_free']
         # Graphics
@@ -1545,25 +1671,25 @@ def output_system_memory_graphs(data):
         # done!
         columndata.append((case, bars, memtext))
     titles = ("Test-case:", "memory usage graph:", swaptext, "RAM used:", "free:")
-    output_memory_graph_table(titles, bar1colors, columndata)
+    output_graph_table(titles, bar1colors, columndata)
     print '<table><tr><th><th align="left">Legend:'
     if testcase['swap_total']:
         print '<tr><td bgcolor="%s" height="16" width="16"><td>Swap used' % bar1colors[0]
     print """
 <tr><td bgcolor="%s" height="16" width="16"><td>RAM used in the device
 <tr><td bgcolor="%s" height="16" width="16"><td>RAM and swap freely usable in the device
-<tr><td bgcolor="%s" height="16" width="16"><td>If memory usage reaches this, application allocations fail and the allocating app is OOM-killed (&gt;= %d MB used)
+<tr><td bgcolor="%s" height="16" width="16"><td>If memory usage reaches this, applications allocations fail and usually they abort as a result (&gt;= %d MB used)
 """ % (bar1colors[1], bar1colors[2], bar1colors[3], round(mem_deny/1024))
     print "</table>"
     if mem_low == mem_total:
         print "<p>(memory limits are not in effect)"
         return
     print """
-<p>Memory usage values which trigger background killing are marked with
-blue color (&gt;= <font color=blue><b>%d</b></font> MB used).<br>
-After bg-killing and memory low mark comes the memory high pressure mark
-at which point e.g.<br> Browser refuses to open new pages, these numbers
-are marked with red color (&gt;= <font color=red><b>%d</b></font> MB used).
+<p>Memory usage values which trigger application background killing and disable their
+pre-starting are marked with blue color (&gt;= <font color=blue><b>%d</b></font> MB used).<br>
+After bg-killing and memory low mark comes the memory high pressure mark at which point
+even pre-started applications are killed and e.g. Browser refuses to open new pages,
+these numbers are marked with red color (&gt;= <font color=red><b>%d</b></font> MB used).
 """ % (round(mem_low/1024), round(mem_high/1024))
 
 
@@ -1584,7 +1710,7 @@ def output_html_report(data):
 <body>
 <h1>%s</h1>
 
-<!-- endurance_report.py v1.1.13 -->
+<!-- endurance_report.py v2.0 -->
 
 <p><b>Contents:</b>
 <ul>
@@ -1593,6 +1719,7 @@ def output_html_report(data):
   <ul>
     <li><a href="#system-memory">System memory usage</a>
     <li><a href="#system-load">System load</a>
+    <li><a href="#network-use">Network usage</a>
     <li><a href="#process-memory">Processes memory usage</a>
   </ul>
 <li>Resource usage changes for each of the test rounds:
@@ -1633,6 +1760,11 @@ def output_html_report(data):
     output_system_load_graphs(data)
     print """
 <hr>
+<a name="network-use"></a>
+<h3>Network usage</h3>"""
+    output_network_use_graphs(data)
+    print """
+<hr>
 <a name="process-memory"></a>
 <h3>Processes memory usage</h3>
 """ # "fool Jed syntax highlighter
@@ -1692,46 +1824,45 @@ def parse_syte_stats(dirs):
     """parses given CSV files into a data structure"""
     data = []
     for dirname in dirs:
+
         # get basic information
-        file = "%s/usage.csv" % dirname
-        sys.stderr.write("Parsing '%s'...\n" % file)
-        items = parse_csv(file)
+        file, filename = syslog.open_compressed("%s/usage.csv" % dirname, syslog.FATAL)
+        items = parse_csv(file, filename)
         if not items:
-            sys.stderr.write("CSV parsing failed\n")
-            sys.exit(1)
+            syslog.error_exit("CSV parsing failed")
 
-        for suffix in ("", ".gz", ".lzo"):
-            file = "%s/smaps.cap%s" % (dirname, suffix)
-            if os.path.exists(file):
-                # get system SMAPS memory usage data
-                sys.stderr.write("Parsing '%s'...\n" % file)
-                items['smaps'], items['private_code'] = parse_smaps(file)
-                if not items['smaps']:
-                    sys.stderr.write("SMAPS data parsing failed\n")
-                    sys.exit(1)
-                break
+        # filename without the extension
+        items['basedir'] = dirname
 
-        for suffix in ("", ".gz", ".lzo"):
-            file = "%s/syslog%s" % (dirname, suffix)
-            if os.path.exists(file):
-                # get the crashes and other errors
-                sys.stderr.write("Parsing '%s'...\n" % file)
-                items['errors'] = syslog.parse_syslog(sys.stdout.write, file)
-                items['logfile'] = file
-                break
-
-        file = "%s/step.txt" % dirname
-        if os.path.exists(file):
+        filename = "%s/step.txt" % dirname
+        if os.path.exists(filename):
             # use-case step description
-            items['description'] = open(file).read().strip()
+            items['description'] = open(filename).read().strip()
 
-        file = "%s/stat" % dirname
-        if os.path.exists(file):
-            sys.stderr.write("Parsing '%s'...\n" % file)
+        file, filename = syslog.open_compressed("%s/smaps.cap" % dirname)
+        if file:
+            # get system SMAPS memory usage data
+            items['smaps'], items['private_code'] = parse_smaps(file)
+            if not items['smaps']:
+                syslog.error_exit("SMAPS data parsing failed")
+
+        file, filename = syslog.open_compressed("%s/syslog" % dirname)
+        if file:
+            # get the crashes and other errors
+            items['logfile'] = filename
+            items['errors'] = syslog.parse_syslog(sys.stdout.write, file)
+
+        file, filename = syslog.open_compressed("%s/stat" % dirname)
+        if file:
             items['/proc/stat'] = parse_proc_stat(file)
             if not items['/proc/stat']:
-                sys.stderr.write("/proc/stat parsing failed\n")
-                sys.exit(1)
+                syslog.error_exit("/proc/stat parsing failed")
+
+        file, filename = syslog.open_compressed("%s/ifconfig" % dirname)
+        if file:
+            items['transfers'] = parse_ifconfig(file)
+            if not items['transfers']:
+                syslog.error_exit("ifconfig output parsing failed")
 
         data.append(items)
     return data
@@ -1740,8 +1871,7 @@ def parse_syte_stats(dirs):
 if __name__ == "__main__":
     if len(sys.argv) < 3:
         msg = __doc__.replace("<TOOL_NAME>", sys.argv[0].split('/')[-1])
-        sys.stderr.write(msg)
-        sys.exit(1)
+        syslog.error_exit(msg)
     # Use psyco if available. Gives 2-3x speed up.
     try:
         import psyco
